__doc__="""ZCatalogedObject.py

Randolpho St. John
N2 The Net
randolphothegreat@yahoo.com

This module contians the Z Caloged Object class and factory registration methods."""

__version__="1.0"

from Products.ZCatalog import CatalogPathAwareness
from OFS import SimpleItem
from OFS.PropertyManager import PropertyManager
from AccessControl import ClassSecurityInfo
from Globals import HTMLFile
from Globals import InitializeClass

# default PropertyManager Permission string:
Permission = "Manage properties"

# zODB factory registration methods:
manage_addZCatalogedObjectForm = HTMLFile('dtml/add', globals())

def manage_addZCatalogedObject(self, id, title="", default_catalog="", ids=[], values=[], types=[], REQUEST=None):
    """create a new Z Cataloged Object and register it with the zODB"""
    self._setObject(id, ZCatalogedObject(id, title, default_catalog, ids, values, types))
    if REQUEST is not None:
        try: url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect("%s/manage_main" % url)
    return ""

class ZCatalogedObject(CatalogPathAwareness.CatalogAware, PropertyManager, SimpleItem.SimpleItem):
    "This class represents an arbitrary store of data that is ZCatalog Aware"

    #
    # Standard Class Setup
    #
    security = ClassSecurityInfo()

    meta_type = "Z Cataloged Object"

    _properties=({"id":"title", "type":"string", "mode":"w"},)

    manage_options=(
        {"label":"Properties", "action":"manage_propertiesForm", "help":("OFSP", "Properties.stx")},
        {"label":"Default Catalog", "action":"manage_editCatalogerForm"}
        )
    
    #
    # Private Methods
    #
    security.declarePrivate("__init__")
    def __init__(self, id, title, default_catalog, ids, values, types):
        """Z Cataloged Object constructor.
        In addition to the standard zope object constructor requirements of id and title,
        this constructor requires:
        
        default_catalog --  the id of the ZCatalog this object wishes to register itself with.
        ids -- a list of the name of the properties to set on this object.
        values -- a list of the values of the properties ot set on this object.
        types -- a list of the data types of the values of the properties to set on this object.
        
        ids, values, and types will be synchronized by index; their lengths must match. """
        
        # validate input
        id = str(id).strip() 
        title = str(title)
        default_catalog = str(default_catalog).strip()

        # default data
        self.default_catalog = default_catalog
        self.id = id
        self.title = title
        
        # match the size of the lists to iterate over
        size = len(ids)
        if len(values) != size:
            raise "List Length Mismatch", "The lengths of ids, values, and types must match"
        if len(types) != size:
            raise "List Length Mismatch", "The lengths of ids, values, and types must match"

        # add the properties; manage_addProperty will validate the data for us.        
        for i in range(size):
            PropertyManager.manage_addProperty(self, ids[i], values[i], types[i], None)
            
    #
    # Management Methods
    #
    security.declareProtected(Permission, "manage_addProperty")
    def manage_addProperty(self, id, value, type, REQUEST=None):
        "override PropertyManager management methods to mixin catalog awareness"
        try:
            PropertyManager.manage_addProperty(self, id, value, type, REQUEST)
        finally:
            self.reindex_object()
        if REQUEST is not None:
            message="Saved changes."
            return self.manage_propertiesForm(self, REQUEST, manage_tabs_message=message)
        
        
    security.declareProtected(Permission, "manage_editProperties")
    def manage_editProperties(self, REQUEST=None):
        "override PropertyManager management methods to mixin catalog awareness"
        try:
            PropertyManager.manage_editProperties(self, REQUEST)
        finally:
            self.reindex_object()
        if REQUEST:
            message="Saved changes."
            return self.manage_propertiesForm(self, REQUEST, manage_tabs_message=message)
        
    security.declareProtected(Permission, "manage_changeProperties")
    def manage_changeProperties(self, REQUEST=None, **kw):
        "override PropertyManager management methods to mixin catalog awareness"
        try:
            PropertyManager.manage_changeProperties(self, REQUEST, kw)
        finally:
            self.reindex_object()
        if REQUEST is not None:
            message="Saved changes."
            return self.manage_propertiesForm(self, REQUEST, manage_tabs_message=message)
        
    security.declareProtected(Permission, "manage_changePropertyTypes")
    def manage_changePropertyTypes(self, old_ids, props, REQUEST=None):
        "override PropertyManager management methods to mixin catalog awareness"
        try:
            toReturn = PropertyManager.manage_changePropertyTypes(self, old_ids, props, REQUEST)
        finally:
            toReturn = ""
            self.reindex_object()
        if REQUEST is not None:
            message="Saved changes."
            return self.manage_propertiesForm(self, REQUEST, manage_tabs_message=message)
        
    security.declareProtected(Permission, "manage_delProperties")
    def manage_delProperties(self, ids=None, REQUEST=None):
        "override PropertyManager management methods to mixin catalog awareness"
        try:
            toReturn = PropertyManager.manage_delProperties(self, ids, REQUEST)
        finally:
            toReturn = ""
            self.reindex_object()
        if REQUEST is not None:
            message="Saved changes."
            return self.manage_propertiesForm(self, REQUEST, manage_tabs_message=message)
     
InitializeClass(ZCatalogedObject)